const BattleCalc = require( '../BattleCalc' );
const Balance = require( '../../constants/Balance' );

const AICalc = {
  SUPPORT_DECISION_LOW_HP_THRESHOLD: 80, // percent
	
	getBestMeleeTarget: function( actor, allEnemies ) {		
		if( allEnemies.length === 0 ) {
			return null;
		}
		
		var bestTarget = allEnemies[0]; 
							
		for( var i = 0; i < allEnemies.length; ++i ) {
			var enemy = allEnemies[ i ];
			if( !enemy.inPlay ) {
				continue;
			}
			
			var dist = BattleCalc.boardDistance( actor, enemy );
			var hpFactor = dist * enemy.hp;
			
			var currentBestDist = BattleCalc.boardDistance( actor, bestTarget );
			var currentBestHPFactor = currentBestDist * bestTarget.hp;
			
			if( hpFactor < currentBestHPFactor ) {
				bestTarget = enemy;
			}
		}
		
		return bestTarget;
	},
	
	getBestMissileTarget: function( actor, candidates )	{			
		return AICalc.getBestMeleeTarget( actor, candidates );
	},
	
	defineEnemyCluster: function( cache, target, candidates ) {
		cache.splice( 0, cache.length );
		
		for( var i = 0; i < candidates.length; ++i ) {
			var enemy = candidates[ i ];
			if( BattleCalc.boardDistance(enemy,target) < 3 ) {
				cache.push( enemy );
			}
		}

		return cache;
	},
	
	getNearbyUnits: function( subject, candidates, defineNearby ) {			
		var nearbyUnits = [];
		for( var i = 0; i < candidates.length; ++i ) {
			var unit = candidates[ i ];
			if( BattleCalc.boardDistance(subject,unit) <= defineNearby ) {
				nearbyUnits.push( unit );
			}
		}
		
		return nearbyUnits;
	},			
	
	getBestSummonTile( actor, battleState )	{
		var tile = null;
		var candidateTiles = [];
		var attempts = 0;
		
		var distFromActor = 1;
		do {
			for( var tid in battleState.allTiles ) {
				var tile = battleState.allTiles[ tid ];
				if( BattleCalc.boardDistance(tile,actor) <= distFromActor && !tile.occupied ) {
					candidateTiles.push( tile );
				}
			}
			++distFromActor;
		} while( distFromActor <= Balance.CONJURATION_RANGE && candidateTiles.length===0 && ++attempts<1000 );
		
		if( candidateTiles.length === 0 ) {
			return null;
		} else {			
			return candidateTiles[ Math.floor(Math.random()*candidateTiles.length) ];
		}
	}
}
module.exports = AICalc
