/**
 * ...
 * @author Robert Lombardo, Wakefield Studios ( rob@wakefield-studios.com )
 */
const Q_how_many_nearby_enemies_BASE = require( '../questions/Q_how_many_nearby_enemies_BASE' );
const Q_can_i_just_kill_it = require( './Q_can_i_just_kill_it' );
const Q_ranged_cluster = require( './Q_ranged_cluster' );
const Q_which_aoe = require( './Q_which_aoe' );
const BattleCalc = require( '../../BattleCalc' );

const InterferenceDecision = function( actor, battleState ) {
	const node = new Q_how_many_nearby_enemies_BASE( actor, battleState );

	// branch indexes
	node.ONE = 1;
	node.TWO_OR_MORE = 2;
	
	node.defineNearby = 3;
	
	node.branches[ node.ONE ] = new Q_can_i_just_kill_it( actor, battleState );
	node.branches[ node.ZERO ] = new Q_ranged_cluster( node.branches[node.ONE], actor, battleState );
	node.branches[ node.TWO_OR_MORE ] = new Q_which_aoe( node.branches[node.ONE], actor, battleState );
	
	node.query = function()	{
		// we prevent the use of 2 cc abilities in a row
		if( this.actor.lastAbilityExecuted ) {
			switch( this.actor.lastAbilityExecuted ) {
				case 'blunt_arrow':
				case 'bash':
				case 'devil_mask':
				case 'ice_nine':
				case 'sky_hammer':
				case 'psyche_out':
				case 'road_to_nowhere':
				case 'leg_shot':
				case 'sleight_of_hand':
					return null;
			}
		}
		
		const numNearby = this.calculateNearbyEnemies( this.defineNearby );

		const bestTarget = this.calculateBestInterferenceTarget();

		if( !bestTarget ) {
			return null;
		}
		
		if( numNearby===0 ) {
			return this.branches[this.ZERO].query( bestTarget );
		} else if( numNearby===1 ) {
			return this.branches[this.ONE].query( bestTarget );
		} else {
			return this.branches[this.TWO_OR_MORE].query( bestTarget );
		}
	};
	
	node.calculateBestInterferenceTarget = function() {	
		var bestTarget = null;
		const allEnemies = this.battleState.getEnemiesOfUnit( this.actor );
		
		if( allEnemies.length===0 ) {
			return null;
		}
		
		const fewestConditions = this.calculateFewestConditions( allEnemies );
		const candidates = this.calculateEnemiesWithFewestConditions( allEnemies, fewestConditions );
		return this.calculateClosestCandidate( candidates ) || allEnemies[ 0 ];
	}
	
	node.getNumCCConditions = function( unit ) {
		if( !unit ) {
			return 99;
		}

		var conditionCount = 0;
		
		for( var prop in unit.conditions ) {
			var c = unit.conditions[ prop ]; 
			if( 	c.handle==='enraged'
				||	c.handle==='scared'
				||	c.handle==='slowed'
				||	c.handle==='stunned' )
			{
				++conditionCount;
			}
		}
		
		return conditionCount;
	}

	node.calculateFewestConditions = function( enemies ) {	
		for( var i = 0; i < 5; ++i ) {
			for( var j = 0; j < enemies.length; ++j ) {
				var enemy = enemies[ j ];

				if( this.getNumCCConditions(enemy)===i ) {
					return i;
				}
			}
		}	
	};	
	
	node.calculateEnemiesWithFewestConditions = function( enemies, fewestConditions ) {
		var candidates = [];
		
		for( var i = 0; i < enemies.length; ++i ) {
			var enemy = enemies[ i ];

			if( this.getNumCCConditions(enemy) <= fewestConditions ) {
				candidates.push( enemy );
			}
		}

		return candidates;
	};
	
	node.calculateClosestCandidate = function( candidates ) {
		if( candidates.length===0) {
			return null;
		}
		
		var bestTarget = candidates[0];
		var bestDist = BattleCalc.boardDistance( this.actor, bestTarget );
		
		for( var i = 0; i < candidates.length; ++i ) {
			var enemy = candidates[ i ];

			var dist = BattleCalc.boardDistance( actor, enemy );
			
			if( dist < bestDist ) {
				bestTarget = enemy;
				bestDist = dist;
			}
		}		

		return bestTarget;
	};

	return node;
}
module.exports = InterferenceDecision
