const TreeNode = require( '../TreeNode' );
const YesNoNode = require( '../YesNoNode' );
const Q_which_jump_ability = require( './Q_which_jump_ability' );
const BattleCalc = require( '../../BattleCalc' );

const Q_target_occupied = function( actor, battleState ) {			
	const node = new YesNoNode( actor, battleState );

	node.leap = BattleCalc.getEquippedAbility( actor, 'leap' );
	node.teleport = BattleCalc.getEquippedAbility( actor, 'teleport' );

	var _bestDist = 99;
	
	node.branches[node.YES] = new TreeNode( null, null ); // we will also use this for the case that the actor is already there.
	node.branches[node.NO] = new Q_which_jump_ability( actor, battleState );			
	
	node.query = function( targetTile ) {
		var posBuf = { x:targetTile.x, y:targetTile.y };

		// var _targetDist = BattleCalc.boardDistance( this.actor, targetTile );
		
		var attempts = 0;
		while( targetTile.occupied && attempts < 32 ) {
			if( 	this.actor.x === targetTile.x
				&&	this.actor.y === targetTile.y )
			{
				// the actor is already there, so don't recurse for more jump tiles
				return this.branches[YES].query();
			}
			
			targetTile = calculateNextClosestTile( this.actor, targetTile );
			
			++attempts;
		}

		// if( posBuf.x===targetTile.x && posBuf.y===targetTile.y ) {
			// return null;
		// } else {
			return this.branches[this.NO].query( targetTile );
		// }
		
		function calculateNextClosestTile( actor, currentTarget ) {		
			var bestTile = currentTarget;
			var bestDist = BattleCalc.boardDistance( actor, bestTile );
			var adjacentTiles = battleState.getAdjacentTiles( currentTarget );
			
			for( var i = 0; i < adjacentTiles.length; ++i ) {
				var tile = adjacentTiles[ i ];
				var tileDist = BattleCalc.boardDistance( actor, tile );
				
				if( !tile.occupied ) {
					if( tileDist <= bestDist ) {
						bestTile = tile;
					}

					if( tileDist < bestDist ) {
						bestDist = tileDist;
					}
				}


			}

			return bestTile;
		}
	};
	
	node.yesNoEvalFunction = function( targetTile ) {
		var result;

		if( this.leap && BattleCalc.getLeapAPCostByDist(this.actor,targetDist) > this.actor.max_ap ) {
			// treat a tile as occupied if it is beyond AP range limitation
			result = true;
		} else if( this.teleport &&	BattleCalc.getTeleportMPCostByDist(this.actor,targetDist) > actor.max_mp ) {
			// treat a tile as occupied if it is beyond MP range limitation
			result = true;
		} else {			
			result = targetTile.occupied;
		}

		return result;
	};		
	return node;
}
module.exports = Q_target_occupied
